<?php

namespace Drupal\file_access_via_webform\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Session\AccountInterface;
use Drupal\file\Entity\File;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;

/**
 * Controller for handling secure file downloads.
 */
class FileDownloadController extends ControllerBase {

  /**
   * Display a file securely in a new tab or download it.
   *
   * @param int $file_id
   *   The file ID to display.
   * @param string $token
   *   The security token.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The file display response.
   */
  public function download($file_id, $token) {
    // Load the file.
    $file = File::load($file_id);
    if (!$file) {
      $this->messenger()->addError($this->t('File not found.'));
      throw new NotFoundHttpException();
    }

    // Check if the file exists on the filesystem.
    $uri = $file->getFileUri();
    if (!file_exists($uri)) {
      $this->messenger()->addError($this->t('File not found on the server.'));
      throw new NotFoundHttpException();
    }

    // Get download settings from session.
    $session = \Drupal::service('session');
    $download_settings = $session->get('file_download_settings_' . $file_id, []);
    $disposition = (isset($download_settings['disposition']) && $download_settings['disposition'] === 'attachment') ? ResponseHeaderBag::DISPOSITION_ATTACHMENT : ResponseHeaderBag::DISPOSITION_INLINE;

    // Clear the settings from session.
    $session->remove('file_download_settings_' . $file_id);

    // Create the response.
    $response = new BinaryFileResponse($uri);
    $response->setContentDisposition(
      $disposition,
      $file->getFilename()
    );

    // Add cache control headers to prevent caching.
    $response->headers->set('Cache-Control', 'no-store, no-cache, must-revalidate, max-age=0');
    $response->headers->set('Pragma', 'no-cache');
    $response->headers->set('Expires', '0');

    return $response;
  }

  /**
   * Access callback for the download route.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The user account.
   * @param int $file_id
   *   The file ID.
   * @param string $token
   *   The security token.
   *
   * @return \Drupal\Core\Access\AccessResult
   *   The access result.
   */
  public function access(AccountInterface $account, $file_id, $token) {
    // Basic permission check.
    if (!$account->hasPermission('access content')) {
      return AccessResult::forbidden();
    }

    // Verify the token.
    if (!$this->verifyToken($file_id, $token)) {
      return AccessResult::forbidden();
    }

    return AccessResult::allowed();
  }

  /**
   * Verify the security token.
   *
   * @param int $file_id
   *   The file ID.
   * @param string $token
   *   The security token.
   *
   * @return bool
   *   TRUE if the token is valid, FALSE otherwise.
   */
  protected function verifyToken($file_id, $token) {
    // Get the stored token from the session.
    $stored_token = \Drupal::service('session')->get('file_download_token_' . $file_id);
    // Clear the token from the session to prevent reuse.
    \Drupal::service('session')->remove('file_download_token_' . $file_id);

    // Compare tokens.
    return $stored_token && hash_equals($stored_token, $token);
  }

}
