<?php

namespace Drupal\file_access_via_webform\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Url;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'webform_download_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "webform_download_formatter",
 *   label = @Translation("Webform Download Button"),
 *   field_types = {
 *     "file",
 *   }
 * )
 */
class WebformDownloadFormatter extends FormatterBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Constructs a WebformDownloadFormatter object.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   */
  public function __construct($plugin_id, $plugin_definition, $field_definition, array $settings, $label, $view_mode, array $third_party_settings, EntityTypeManagerInterface $entity_type_manager, MessengerInterface $messenger, ModuleHandlerInterface $module_handler) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->entityTypeManager = $entity_type_manager;
    $this->messenger = $messenger;
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('entity_type.manager'),
      $container->get('messenger'),
      $container->get('module_handler')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'modal_width' => '800px',
      'dialog_show_header' => 1,
      'dialog_show_header_title' => 0,
      'webform_id' => 'contact',
      'download_link_text' => 'Download',
      'bootstrap4_modal_enable' => 0,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);

    $elements['modal_width'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Modal width'),
      '#default_value' => $this->getSetting('modal_width'),
      '#description' => $this->t('Width of the modal window (e.g. 800px, 90%)'),
      '#required' => TRUE,
    ];

    $elements['download_link_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Download link text'),
      '#default_value' => $this->getSetting('download_link_text'),
      '#description' => $this->t('Text to display on the download button.'),
      '#required' => TRUE,
    ];

    // Only show these options if Bootstrap 4 Modal module is enabled.
    if ($this->moduleHandler->moduleExists('bootstrap4_modal')) {

      // Get the current field name for use in states conditions.
      $field_name = $this->fieldDefinition->getName();
      $elements['bootstrap4_modal_enable'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Enable Bootstrap 4 Modal'),
        '#default_value' => $this->getSetting('bootstrap4_modal_enable'),
        '#description' => $this->t('Enable Bootstrap 4 Modal integration for the download form. If you are using a non-Bootstrap theme, this will not work.'),
      ];

      $elements['dialog_show_header'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Show dialog header'),
        '#default_value' => (bool) $this->getSetting('dialog_show_header'),
        '#description' => $this->t('Whether to show the dialog header.'),
        '#states' => [
          'visible' => [
            ':input[name="fields[' . $field_name . '][settings_edit_form][settings][bootstrap4_modal_enable]"]' => ['checked' => TRUE],
          ],
        ],
      ];

      $elements['dialog_show_header_title'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Show dialog header title'),
        '#default_value' => (bool) $this->getSetting('dialog_show_header_title'),
        '#description' => $this->t('Whether to show the title in the dialog header.'),
        '#states' => [
          'visible' => [
            ':input[name="fields[' . $field_name . '][settings_edit_form][settings][bootstrap4_modal_enable]"]' => ['checked' => TRUE],
          ],
        ],
      ];
    }

    // Load all webforms for selection.
    $webforms = $this->entityTypeManager->getStorage('webform')->loadMultiple();
    $options = [];
    foreach ($webforms as $webform) {
      $options[$webform->id()] = $webform->label();
    }

    $elements['webform_id'] = [
      '#type' => 'select',
      '#title' => $this->t('Webform'),
      '#default_value' => $this->getSetting('webform_id'),
      '#options' => $options,
      '#description' => $this->t('Select the webform to use for the download process.'),
      '#required' => TRUE,
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();
    $summary[] = $this->t('Modal width: @width', ['@width' => $this->getSetting('modal_width')]);
    $summary[] = $this->t('Download link text: @text', ['@text' => $this->getSetting('download_link_text')]);
    if ($this->moduleHandler->moduleExists('bootstrap4_modal') && $this->getSetting('bootstrap4_modal_enable') == 1) {
      $summary[] = $this->t('Show dialog header: @show', ['@show' => (bool) $this->getSetting('dialog_show_header') ? 'Yes' : 'No']);
      $summary[] = $this->t('Show dialog header title: @show', ['@show' => (bool) $this->getSetting('dialog_show_header_title') ? 'Yes' : 'No']);
    }
    $summary[] = $this->t('Webform: @webform', ['@webform' => $this->getSetting('webform_id')]);
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    $webform_id = $this->getSetting('webform_id');

    // Check if webform is selected.
    if (empty($webform_id)) {
      $this->messenger->addWarning($this->t('Please select a webform in the field formatter settings.'));
      return $elements;
    }

    // Verify webform exists.
    $webform = $this->entityTypeManager->getStorage('webform')->load($webform_id);
    if (!$webform) {
      $this->messenger->addError($this->t('Selected webform "@webform" does not exist.', ['@webform' => $webform_id]));
      return $elements;
    }

    // Check if Bootstrap 4 Modal is available.
    $use_bootstrap_modal = $this->moduleHandler->moduleExists('bootstrap4_modal') && $this->getSetting('bootstrap4_modal_enable') == 1;

    foreach ($items as $delta => $item) {
      $file = NULL;

      // If this is a media reference, get the file entity.
      if ($item->entity && $item->entity->bundle() === 'document') {
        $file = $item->entity->get('field_media_file')->entity ?? NULL;
      }
      elseif ($item->entity && $item->entity->getEntityTypeId() === 'file') {
        $file = $item->entity;
      }

      if ($file) {
        $webform_url = Url::fromRoute('entity.webform.canonical', ['webform' => $webform_id], [
          'query' => ['source_entity_type' => 'file', 'source_entity_id' => $file->id()],
          'attributes' => [
            'class' => ['use-ajax', 'button', 'button--primary'],
            'data-dialog-type' => 'modal',
          ],
        ]);

        $elements[$delta] = [
          '#type' => 'link',
          '#title' => $this->getSetting('download_link_text'),
          '#url' => $webform_url,
          '#attributes' => [
            'class' => ['use-ajax', 'download-link', 'btn', 'primary-btn'],
            'data-dialog-type' => $use_bootstrap_modal ? 'bootstrap4_modal' : 'modal',
          ],
        ];

        // Add Bootstrap 4 Modal specific options if available.
        if ($use_bootstrap_modal) {
          $elements[$delta]['#attributes']['data-dialog-options'] = json_encode([
            'dialogClasses' => 'modal-dialog-centered modal-lg',
            'dialogShowHeader' => $this->getSetting('dialog_show_header') == 1 ? TRUE : FALSE,
            'dialogShowHeaderTitle' => $this->getSetting('dialog_show_header_title') == 1 ? TRUE : FALSE,
            'width' => $this->getSetting('modal_width'),
          ]);
        }
        else {
          // Default Drupal modal options.
          $elements[$delta]['#attributes']['data-dialog-options'] = json_encode([
            'width' => $this->getSetting('modal_width'),
            'dialogClass' => 'webform-download-dialog',
          ]);
        }
      }
    }

    return $elements;
  }

}
